/************************************************/
/* uPD7210 driver				*/
/*----------------------------------------------*/
/* main body   					*/
/*						*/
/* This file is completely in public domain.	*/
/************************************************/
/*
 $Id: upd7210.c,v 1.8 2003/11/03 12:40:48 hirohito Exp $
*/

/***** Feature test switches ************************************************/
/***** System headers *******************************************************/
#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/errno.h>
#include <sys/conf.h>
#include <sys/module.h>
#include <sys/uio.h>
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>
#include <machine/clock.h>
#include <sys/proc.h>


/***** Local headers ********************************************************/
#include <dev/upd7210/upd7210.h>
#include <dev/upd7210/upd7210_var.h>


/***** Depend on expansion board ********************************************/
/***** Constat values *******************************************************/
/***** Macros ***************************************************************/
/***** Typedefs *************************************************************/
/***** Function prototypes **************************************************/
static int  wait_if_other_use( GPIB_SOFTC *psc );
/* static void  gpib_identify( driver_t *driver, device_t parent ); */
static void  gpib_atnon( GPIB_SOFTC *psc );
static void  gpib_atnoff( GPIB_SOFTC *psc );
static int  gpib_ifc( GPIB_SOFTC *psc );
static int  gpib_remote( GPIB_SOFTC *psc );
static int  gpib_local( GPIB_SOFTC *psc );
static int  gpib_comout( GPIB_SOFTC *psc, char *cmd, int msglen );
static int  gpib_cmd( GPIB_SOFTC *psc, int cmd );
static int  gpib_cmd_wl( GPIB_SOFTC *psc, int listener[], int cmd );
static int  gpib_sta( GPIB_SOFTC *psc, int talker );
static int  gpib_sla( GPIB_SOFTC *psc, int listener[] );
static int  gpib_talk( GPIB_SOFTC *psc, GPIB_PACKET *ppkt );
static int  gpib_listen( GPIB_SOFTC *psc, GPIB_PACKET *ppkt );
static int  gpib_sp( GPIB_SOFTC *psc, GPIB_PACKET *ppkt );
static int  upd7210_command( GPIB_SOFTC *psc );
static int  upd7210_talk( GPIB_SOFTC *psc );
static int  upd7210_listen( GPIB_SOFTC *psc );
static int  upd7210_initialize( GPIB_SOFTC *psc );


/***** device driver information ********************************************/
devclass_t gpib_devclass;

static d_open_t  gpib_open;
static d_close_t  gpib_close;
static d_read_t  gpib_read;
static d_write_t  gpib_write;
static d_ioctl_t  gpib_ioctl;

static struct cdevsw  gpib_cdevsw =
{
    gpib_open,		/* open */
    gpib_close,		/* close */
    gpib_read,		/* read */
    gpib_write,		/* write */
    gpib_ioctl,		/* ioctl */
    nopoll,
    nommap,
    nostrategy,
    DRIVER_NAME,	/* pointer of driver name */
    CDEV_MAJOR,
    nodump,
    nopsize,
    0,			/* flags */
    -1,			/* bmaj */
};


/***** Local variables ******************************************************/
/***** Global variables *****************************************************/

/***** Local functions ******************************************************/
/*==========================================================================
 * ¾
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	0ｪλ
 *--------------------------------------------------------------------------
 * (note)
 *  渢줿顢psc->inuse򥻥åȤƵ롣
*/
static int  wait_if_other_use( GPIB_SOFTC *psc )
{
    intrmask_t ipl = splhigh();
    while( psc->inuse )
    {
	int ret = tsleep( (void*)&psc->inuse, PWAIT|PCATCH, "pwait", 0 );
	if( ret != 0 )
	{
	    splx( ipl );
	    return ret;
	}
    }
    psc->inuse = 1;
    splx( ipl );
    return 0;
}



/***** Global functions *****************************************************/
/*==========================================================================
 * OPEN
 *--------------------------------------------------------------------------
 * arg: dev
 *	flag
 *	mode
 *	p
 * ret: int	
 *--------------------------------------------------------------------------
*/
static int  gpib_open( dev_t dev, int flag, int mode, struct proc *p )
{
    int unit, gpibadrs;

    unit = minor(dev)/32;		/* GPIB maximum address is 30 */
    gpibadrs = minor(dev)%32;
#ifdef GPIB_DEBUG
    printf( "==GPIB== open() unit=%d address=%d\n", unit, gpibadrs );
#endif
/*    if( unit >= NGPIB ) return ENXIO; */
    if( gpibadrs > GPIB_MAX_ADDRESS ) return ENXIO;
    return 0;
}



/*==========================================================================
 * CLOSE
 *--------------------------------------------------------------------------
 * arg: dev
 *	flag
 *	mode
 *	p
 * ret: int	
 *--------------------------------------------------------------------------
*/
static int  gpib_close( dev_t dev, int flag, int mode, struct proc *p )
{
    int unit, gpibadrs;

    unit = minor(dev)/32;		/* GPIB maximum address is 30 */
    gpibadrs = minor(dev)%32;
#ifdef GPIB_DEBUG
    printf( "==GPIB== close() unit=%d address=%d\n", unit, gpibadrs );
#endif

/*    if( unit >= NGPIB ) return ENXIO;*/
    if( gpibadrs > GPIB_MAX_ADDRESS ) return ENXIO;

    return 0;
}



/*==========================================================================
 * READ
 *--------------------------------------------------------------------------
 * arg: dev
 *	uio
 *	flag
 * ret: int	
 *--------------------------------------------------------------------------
*/
static int  gpib_read( dev_t dev, struct uio *uio, int flag )
{
    GPIB_SOFTC *psc;
    int unit, gpibadrs;
    int ret, len;

    unit = minor(dev)/32;		/* GPIB maximum address is 30 */
    gpibadrs = minor(dev)%32;
#ifdef GPIB_DEBUG
    printf( "==GPIB== read() unit=%d address=%d\n", unit, gpibadrs );
#endif

/*    if( unit >= NGPIB ) return ENXIO;*/
    if( gpibadrs > GPIB_MAX_ADDRESS ) return ENXIO;
    psc = devclass_get_softc( gpib_devclass, unit );
    if( (ret = wait_if_other_use( psc )) != 0 ) return ret;

    ret = EIO;
    psc->delimiter = psc->default_delimiter;
    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end++ = UNL;
    *psc->pbuf_end++ = gpibadrs | TAG;
    *psc->pbuf_end   = psc->myaddress | LAG;
    if( upd7210_command( psc ) != GPIB_NOERR ) goto RETURN;

    psc->pbuf = psc->buf;
    psc->pbuf_end = psc->pbuf + sizeof(psc->buf) - 1;
    switch( upd7210_listen( psc ) )
    {
    case GPIB_NOERR:
    case GPIB_EBFULL:
	len = psc->pbuf - psc->buf;
	uiomove( psc->buf, uio->uio_resid < len? uio->uio_resid: len, uio );
	break;
    case GPIB_ETIME:			/* timeout */
	break;
    default:
	goto RETURN;
    }

    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end   = UNL;
    if( upd7210_command( psc ) != GPIB_NOERR ) goto RETURN;
    ret = 0;

RETURN:
    gpib_atnon( psc );
    psc->inuse = 0;
    wakeup( (void*)&psc->inuse );
    return ret;

    return 0;
}



/*==========================================================================
 * WRITE
 *--------------------------------------------------------------------------
 * arg: dev
 *	uio
 *	flag
 * ret: int	
 *--------------------------------------------------------------------------
*/
static int  gpib_write( dev_t dev, struct uio *uio, int flag )
{
    GPIB_SOFTC *psc;
    int unit, gpibadrs;
    int ret;

    unit = minor(dev)/32;		/* GPIB maximum address is 30 */
    gpibadrs = minor(dev)%32;
#ifdef GPIB_DEBUG
    printf( "==GPIB== write() unit=%d address=%d\n", unit, gpibadrs );
#endif

/*    if( unit >= NGPIB ) return ENXIO;*/
    if( gpibadrs > GPIB_MAX_ADDRESS ) return ENXIO;
    psc = devclass_get_softc( gpib_devclass, unit );
    if( uio->uio_resid > GPIB_BUFFERSIZE ) return ENOMEM;
    if( (ret = wait_if_other_use( psc )) != 0 ) return ret;

    ret = EIO;
    psc->delimiter = psc->default_delimiter;
    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end++ = UNL;
    *psc->pbuf_end++ = psc->myaddress | TAG;
    *psc->pbuf_end   = gpibadrs | LAG;
    if( upd7210_command( psc ) != GPIB_NOERR ) goto RETURN;

    psc->pbuf = psc->buf;
    psc->pbuf_end = psc->pbuf + uio->uio_resid - 1;
    uiomove( psc->buf, uio->uio_resid, uio );
    if( upd7210_talk( psc ) != GPIB_NOERR ) goto RETURN;

    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end   = UNL;
    if( upd7210_command( psc ) != GPIB_NOERR ) goto RETURN;
    ret = 0;

RETURN:
    gpib_atnon( psc );
    psc->inuse = 0;
    wakeup( (void*)&psc->inuse );
    return ret;

    return 0;
}



/*==========================================================================
 * IOCTL
 *--------------------------------------------------------------------------
 * arg: dev
 *	cmd
 *	data
 *	flag
 *	p
 * ret: int	
 *--------------------------------------------------------------------------
*/
static int  gpib_ioctl( dev_t dev, u_long cmd, caddr_t data, int flag, struct proc *p )
{
    GPIB_PACKET *ppkt;
    GPIB_SOFTC *psc;
    int unit, gpibadrs;
    int ret = 0;

    unit = minor(dev)/32;		/* GPIB maximum address is 30 */
    gpibadrs = minor(dev)%32;

/*    if( unit >= NGPIB ) return ENXIO;*/
    if( gpibadrs > GPIB_MAX_ADDRESS ) return ENXIO;
    psc = devclass_get_softc( gpib_devclass, unit );

    ppkt = (GPIB_PACKET*)data;
    if( (ret = wait_if_other_use( psc )) != 0 ) return ret;

    switch( cmd )
    {
    case GPIB_DELIM:
	psc->delimiter = (GPIB_DLM_CR|GPIB_DLM_LF|GPIB_DLM_EOI) & *((int *)data);
	break;
	
    case GPIB_TIMEOUT:
	psc->timeout = *((int *)data);
	break;

    case GPIB_ATNON:
	if( !FLAGS_IS_CONTROLLER(psc->flags) )
	    ppkt->errno = GPIB_ECTRL;
	else
	{
	    gpib_atnon( psc );
	    ppkt->errno = GPIB_NOERR;
	}
	break;
	
    case GPIB_ATNOFF:
	if( !FLAGS_IS_CONTROLLER(psc->flags) )
	    ppkt->errno = GPIB_ECTRL;
	else
	{
	    gpib_atnoff( psc );
	    ppkt->errno = GPIB_NOERR;
	}
	break;

    case GPIB_IFC:	ppkt->errno = gpib_ifc( psc ); break;
    case GPIB_REMOTE:	ppkt->errno = gpib_remote( psc ); break;
    case GPIB_LOCAL:	ppkt->errno = gpib_local( psc ); break;
    case GPIB_COMMAND:	ppkt->errno = gpib_comout( psc, ppkt->message, ppkt->msglen ); break;
    case GPIB_UNT:	ppkt->errno = gpib_cmd( psc, UNT ); break;
    case GPIB_UNL:	ppkt->errno = gpib_cmd( psc, UNL ); break;
    case GPIB_STA:	ppkt->errno = gpib_sta( psc, ppkt->talker ); break;
    case GPIB_SLA:	ppkt->errno = gpib_sla( psc, ppkt->listener ); break;
    case GPIB_DCL:	ppkt->errno = gpib_cmd( psc, DCL ); break;
    case GPIB_GTL:	ppkt->errno = gpib_cmd_wl( psc, ppkt->listener, GTL ); break;
    case GPIB_LLO:	ppkt->errno = gpib_cmd( psc, LLO ); break;
    case GPIB_SDC:	ppkt->errno = gpib_cmd_wl( psc, ppkt->listener, SDC ); break;
    case GPIB_GET:	ppkt->errno = gpib_cmd_wl( psc, ppkt->listener, GET ); break;
    case GPIB_TALK:	ppkt->errno = gpib_talk( psc, ppkt ); break;
    case GPIB_LISTEN:	ppkt->errno = gpib_listen( psc, ppkt ); break;
    case GPIB_SP:	ppkt->errno = gpib_sp( psc, ppkt ); break;
    case GPIB_REQ:
    }

    psc->inuse = 0;
    wakeup( (void*)&psc->inuse );
    return ret;

    return 0;
}



/*==========================================================================
 * Interrupt routine
 *--------------------------------------------------------------------------
 * arg: p	a pointer of softc
 * ret: none
 *--------------------------------------------------------------------------
 * 
*/
void  gpib_interrupt( void *p )
{
    int is1, is2, data = 0;
    GPIB_SOFTC *psc = p;

    is1 = interrupt_status1( psc );
    is2 = interrupt_status2( psc );
    if( is1 & BIT_DI ) data = data_in( psc );

    switch( psc->state )
    {
    case STATE_NONE:
	break;

    case STATE_TALK:
	if( !( is1 & BIT_DO ) ) break;			/* error? */
	if( psc->pbuf < psc->pbuf_end )			/* ǡ */
	{
	    byte_out( psc, *psc->pbuf++ );
	    break;
	}

	if( psc->pbuf == psc->pbuf_end )		/* ǽǡ? */
	{
	    if( psc->delimiter == GPIB_DLM_EOI ) aux_command( psc, SEND_EOI );
	    byte_out( psc, *psc->pbuf++ );
	    break;
	}

	/* ǡФäơǥߥղä뤫ȽǤ롣 */
	psc->state = STATE_EOS1;
	switch( psc->delimiter & (GPIB_DLM_CR|GPIB_DLM_LF) )
	{
	case GPIB_DLM_CR:		/* CR only */
	    if( psc->delimiter & GPIB_DLM_EOI ) aux_command( psc, SEND_EOI );
	    byte_out( psc, 0x0d );
	    break;

	case GPIB_DLM_LF:		/* LF only */
	    if( psc->delimiter & GPIB_DLM_EOI ) aux_command( psc, SEND_EOI );
	    byte_out( psc, 0x0a );
	    break;

	case GPIB_DLM_CR|GPIB_DLM_LF:	/* CR+LF (see below) */
	    byte_out( psc, 0x0d );
	    break;

	default:
	    psc->state = STATE_NONE;
	    break;
	}
	break;

    case STATE_EOS1:
	if( !( is1 & BIT_DO ) ) break;			/* error ? */
	if( (psc->delimiter & (GPIB_DLM_CR|GPIB_DLM_LF))
	    == (GPIB_DLM_CR|GPIB_DLM_LF) )		/* CR+LF */
	{
	    psc->state = STATE_EOS2;
	    if( psc->delimiter & GPIB_DLM_EOI ) aux_command( psc, SEND_EOI );
	    byte_out( psc, 0x0a );
	    break;
	}
	psc->state = STATE_NONE;
	break;

    case STATE_EOS2:
	if( !( is1 & BIT_DO ) ) break;			/* error ? */
	psc->state = STATE_NONE;
	break;
	
    case STATE_LISTEN:
	if( !( is1 & BIT_DI ) ) break;			/* error ? */
	if( psc->pbuf > psc->pbuf_end )			/* buffer full !! */
	{
	    psc->state = STATE_NONE;
	    break;
	}
	*psc->pbuf++ = data;
	if( is1 & BIT_END ) psc->state = STATE_NONE;
	break;

    case STATE_COMMAND:
	if( !( is2 & BIT_CO ) ) break;			/* error ? */
	if( psc->pbuf <= psc->pbuf_end )
	    byte_out( psc, *psc->pbuf++ );
	else
	    psc->state = STATE_NONE;
	break;

    case STATE_POLL:
	if( !( is1 & BIT_DI ) ) break;			/* error ? */
	*psc->pbuf = data;
	psc->state = STATE_NONE;
	break;
    }

#if defined(GPIB_CONTEC_PC)
    bus_space_write_1( psc->bst2, psc->bsh2, psc->int_mask_ofs, 0 );
#endif

    if( psc->state == STATE_NONE ) wakeup( psc );
}


/***** uPD7210 - GPIB functions *********************************************/

/*==========================================================================
 * Activate ATTENTION uniline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: none
 *--------------------------------------------------------------------------
 * (Bus Activity)
 *   ATN
*/
static void  gpib_atnon( GPIB_SOFTC *psc )
{
    int i;

    if( psc->atn ) return;

    aux_command( psc, TAKE_CONTROL_ASYNCHRONOUSLY );
    psc->atn = 1;
    for( i = 0; i < 1000; i++ )
    {
	if( !(address_status( psc ) & BIT_ATN) ) return;
	DELAY( 100 );
	aux_command( psc, TAKE_CONTROL_ASYNCHRONOUSLY );
    }
    printf( "uPD7210-GPIB: Can't activate ATN line.\n" );
}



/*==========================================================================
 * Inactivate ATTENTION uniline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: none
 *--------------------------------------------------------------------------
 * (Bus Activity)
 *   /ATN
*/
static void  gpib_atnoff( GPIB_SOFTC *psc )
{
    int i;

    if( ! psc->atn ) return;

    aux_command( psc, GO_TO_STANDBY );
    psc->atn = 0;
    for( i = 0; i < 1000; i++ )
    {
	if( address_status( psc ) & BIT_ATN ) return;
	DELAY( 100 );
	aux_command( psc, GO_TO_STANDBY );
    }
    printf( "uPD7210-GPIB: Can't inactivate ATN line.\n" );
}



/*==========================================================================
 * INTERFACE CLEAR uniline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus Activity)
 *  ATN - IFC
 *
 * (note)
 * In standard, assert IFC line 100uS or more.
*/
static int  gpib_ifc( GPIB_SOFTC *psc )
{
    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;

    aux_command( psc, TAKE_CONTROL_ASYNCHRONOUSLY );
    psc->atn = 1;

    aux_command( psc, SET_IFC );
    DELAY( 100 );
    aux_command( psc, CLEAR_IFC );

    return GPIB_NOERR;
}



/*==========================================================================
 * Activate REMOTE uniline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus Activity)
 *  REN
*/
static int  gpib_remote( GPIB_SOFTC *psc )
{
    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;
    aux_command( psc, SET_REN );
    return GPIB_NOERR;
}



/*==========================================================================
 * Inactivate REMOTE uniline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus Activity)
 *  /REN
*/
static int  gpib_local( GPIB_SOFTC *psc )
{
    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;
    aux_command( psc, CLEAR_REN );
    return GPIB_NOERR;
}



/*==========================================================================
 * Multiline message (for user)
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - (command...)
*/
static int  gpib_comout( GPIB_SOFTC *psc, char *cmd, int msglen )
{
    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;

    if( msglen <= 0 ) return GPIB_ERANGE;
    if( msglen > GPIB_BUFFERSIZE ) return GPIB_EBFULL;
    memcpy( psc->buf, cmd, msglen );
    psc->pbuf_end = (psc->pbuf = psc->buf) + msglen - 1;
    return upd7210_command( psc );
}



/*==========================================================================
 * Multiline message (1 command)
 *--------------------------------------------------------------------------
 * arg: psc
 *	cmd	command message
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - cmd
*/
static int  gpib_cmd( GPIB_SOFTC *psc, int cmd )
{
    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;

    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end = cmd;
    return upd7210_command( psc );
}



/*==========================================================================
 * Multiline message with listener (listeners and 1 command)
 *--------------------------------------------------------------------------
 * arg: psc
 *	listener[]	listener addresses
 *	cmd		command message
 * ret: int		gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - UNL - LA... - cmd - UNL
*/
static int  gpib_cmd_wl( GPIB_SOFTC *psc, int listener[], int cmd )
{
    int i;

    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;

    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNL;
    for( i = 0; i < GPIB_MAX_DEVICE; i++ )
    {
	if( listener[i] < 0 ) break;
	if( listener[i] > GPIB_MAX_ADDRESS ) return GPIB_EADDR;
	*psc->pbuf_end++ = (listener[i] | LAG);
    }
    if( i == 0 ) return GPIB_EADDR;

    *psc->pbuf_end++ = cmd;
    *psc->pbuf_end   = UNL;
    return upd7210_command( psc );
}



/*==========================================================================
 * Set talker address multiline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - TA
*/
static int  gpib_sta( GPIB_SOFTC *psc, int talker )
{
    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;
    if( (talker < 0) || (talker > GPIB_MAX_ADDRESS) ) return GPIB_EADDR;
    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end = (talker | TAG);
    return upd7210_command( psc );
}



/*==========================================================================
 * Set listener address multiline message
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - LA ...
*/
static int  gpib_sla( GPIB_SOFTC *psc, int listener[] )
{
    int i;

    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;
    psc->pbuf = psc->pbuf_end = psc->buf;
    for( i = 0; i < GPIB_MAX_DEVICE; i++ )
    {
	if( listener[i] < 0 ) break;
	if( listener[i] > GPIB_MAX_ADDRESS ) return GPIB_EADDR;
	*psc->pbuf_end++ = (listener[i] | LAG);
    }
    if( i == 0 ) return GPIB_EADDR;

    psc->pbuf_end--;
    return upd7210_command( psc );
}



/*==========================================================================
 * Talk ( data send )
 *--------------------------------------------------------------------------
 * arg: psc
 *	ppkt	gpib message packet (see upd7210.h)
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - UNL - UNT - TAD(me) - LAD... - data... - UNT - UNL
*/
static int  gpib_talk( GPIB_SOFTC *psc, GPIB_PACKET *ppkt )
{
    int i, err;

    if( ppkt->msglen <= 0 ) return GPIB_ERANGE;
    if( ppkt->msglen > GPIB_BUFFERSIZE ) return GPIB_EBFULL;

    psc->delimiter = ppkt->delimiter;
    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end++ = UNL;
    *psc->pbuf_end = psc->myaddress | TAG;
    for( i = 0; i < GPIB_MAX_DEVICE; i++ )
    {
	if( ppkt->listener[i] < 0 ) break;
	if( ppkt->listener[i] > GPIB_MAX_ADDRESS ) return GPIB_EADDR;
	*++psc->pbuf_end = (ppkt->listener[i] | LAG);
    }
    if( i == 0 ) return GPIB_EADDR;
    if( (err = upd7210_command( psc )) != GPIB_NOERR ) goto RETURN;

    memcpy( psc->buf, ppkt->message, ppkt->msglen );
    psc->pbuf_end = (psc->pbuf = psc->buf) + ppkt->msglen - 1;
    if( (err = upd7210_talk( psc )) != GPIB_NOERR ) goto RETURN;

    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end   = UNL;
    if( (err = upd7210_command( psc )) != GPIB_NOERR ) goto RETURN;
    err = GPIB_NOERR;

RETURN:
    gpib_atnon( psc );
    return err;
}



/*==========================================================================
 * Listen ( data receive )
 *--------------------------------------------------------------------------
 * arg: psc
 *	ppkt	gpib message packet (see upd7210.h)
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN - UNL - UNT - TAD - LAD(me) - data... - UNT - UNL
*/
static int  gpib_listen( GPIB_SOFTC *psc, GPIB_PACKET *ppkt )
{
    int err, err2, len;

    if( (ppkt->talker < 0) || (ppkt->talker > GPIB_MAX_ADDRESS) ) return GPIB_EADDR;

    psc->delimiter = ppkt->delimiter;
    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end++ = UNL;
    *psc->pbuf_end++ = ppkt->talker | TAG;
    *psc->pbuf_end = psc->myaddress | LAG;
    if( (err = upd7210_command( psc )) != GPIB_NOERR ) goto RETURN;

    psc->pbuf = psc->buf;
    psc->pbuf_end = psc->pbuf + sizeof(psc->buf) - 1;
    if( (err = upd7210_listen( psc )) != GPIB_NOERR ) goto RETURN;

    len = psc->pbuf - psc->buf;
    if( ppkt->msglen >= len )
	ppkt->msglen = len;
    else
	err = GPIB_EBFULL;
    memcpy( ppkt->message, psc->buf, ppkt->msglen );

    psc->pbuf = psc->pbuf_end = psc->buf;
    *psc->pbuf_end++ = UNT;
    *psc->pbuf_end   = UNL;
    if( (err2 = upd7210_command( psc )) != GPIB_NOERR && err == GPIB_NOERR ) err = err2;

RETURN:
    gpib_atnon( psc );
    return err;
}



/*==========================================================================
 * Serial poll
 *--------------------------------------------------------------------------
 * arg: psc
 *	ppkt	gpib message packet (see upd7210.h)
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * (Bus sequence)
 *  ATN-SPE-TA1-/ATN-(stb)-ATN-TA2- .... - SPD
*/
static int  gpib_sp( GPIB_SOFTC *psc, GPIB_PACKET *ppkt )
{
    int i, err1, err2;
    intrmask_t ipl;

    if( !FLAGS_IS_CONTROLLER(psc->flags) ) return GPIB_ECTRL;

    auxiliary_mode( psc, 0x81 );	/* Aux(A), RFD Hold off on all data. */
    aux_command( psc, LISTEN );
    *(psc->pbuf = psc->pbuf_end = psc->buf) = SPE;
    if( (err1 = upd7210_command( psc )) != GPIB_NOERR ) goto RETURN;

    for( i = 0; i < GPIB_MAX_DEVICE; i++ )
    {
	int talker = ppkt->listener[i];
	if( talker < 0 ) break;
	if( talker > GPIB_MAX_ADDRESS )
	{
	    ppkt->listener[i] = GPIB_EADDR;
	    continue;
	}
	*psc->buf = (talker | TAG);
	psc->pbuf = psc->pbuf_end = psc->buf;
	if( (err1 = upd7210_command( psc )) != GPIB_NOERR )
	{
	    ppkt->listener[i] = err1;
	    continue;
	}

	psc->pbuf = psc->buf;
	ipl = splhigh();
	psc->state = STATE_POLL;
	gpib_atnoff( psc );
	if( tsleep( psc, PRIBIO, "uPDpol", psc->timeout*hz ) != 0 )
	    ppkt->listener[i] = GPIB_ETIME;
	else
	    ppkt->listener[i] = *psc->buf;
	splx( ipl );
	gpib_atnon( psc );
	aux_command( psc, FINISH_HANDSHAKE );
    }

RETURN:
    *(psc->pbuf = psc->pbuf_end = psc->buf) = SPD;
    err2 = upd7210_command( psc );
    aux_command( psc, LOCAL_UNLISTEN );
    auxiliary_mode( psc, 0x80 );	/* Aux(A), Normal handshake mode. */

    return (err1 != GPIB_NOERR)? err1: err2;
}



/*==========================================================================
 * ޥ饤󥳥ޥɤȯ
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * softcꤵ줿ޥ饤󥳥ޥɤȯԤ
*/
static int  upd7210_command( GPIB_SOFTC *psc )
{
    intrmask_t ipl;
    int ret = GPIB_NOERR;
    
    ipl = splhigh();
    psc->state = STATE_COMMAND;
    gpib_atnon( psc );
    byte_out( psc, *psc->pbuf++ );
    if( tsleep( psc, PRIBIO, "uPDcmd", psc->timeout*hz ) != 0 ) ret = GPIB_ETIME;
    splx( ipl );
#ifdef GPIB_DEBUG
    if( ret == GPIB_ETIME ) printf( "==GPIB== command(): Timeout occurred.\n" );
#endif

    return ret;
}



/*==========================================================================
 * ǡ
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * softcꤵ줿ȳߥ롼Ȥäơ򳫻Ϥ
 * ߤϡޥ⡼ɤΤ߼
*/
static int  upd7210_talk( GPIB_SOFTC *psc )
{
    intrmask_t ipl;
    int ret = GPIB_NOERR;

    ipl = splhigh();
    psc->state = STATE_TALK;
    gpib_atnoff( psc );
    if( tsleep( psc, PRIBIO, "uPDtlk", psc->timeout*hz ) != 0 ) ret = GPIB_ETIME;
    splx( ipl );
#ifdef GPIB_DEBUG
    if( ret == GPIB_ETIME ) printf( "==GPIB== talk(): Timeout occurred.\n" );
#endif

    return ret;
}



/*==========================================================================
 * ǡ
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
 * softcꤵ줿ȳߥ롼Ȥäơ򳫻Ϥ롣
 * ߤϡޥ⡼ɤΤ߼
*/
static int  upd7210_listen( GPIB_SOFTC *psc )
{
    intrmask_t ipl;
    int ret = GPIB_NOERR;

    switch( psc->delimiter & (GPIB_DLM_CR|GPIB_DLM_LF) )
    {
    case GPIB_DLM_CR:
	auxiliary_mode( psc, 0x80 | 0x14 );	/* Aux (A), set END bit with EOS message */
	end_of_string( psc, 0x0d );
	break;

    case GPIB_DLM_LF:
    case GPIB_DLM_CR|GPIB_DLM_LF:
	auxiliary_mode( psc, 0x80 | 0x14 );	/* Aux (A), set END bit with EOS message */
	end_of_string( psc, 0x0a );
	break;

    default:
	auxiliary_mode( psc, 0x80 | 0x10 );	/* Aux (A), no set END bit with EOS message */
    }

    ipl = splhigh();
    psc->state = STATE_LISTEN;
    gpib_atnoff( psc );
    if( tsleep( psc, PRIBIO, "uPDlsn", psc->timeout*hz ) != 0 ) ret = GPIB_ETIME;
    splx( ipl );
#ifdef GPIB_DEBUG
    if( ret == GPIB_ETIME ) printf( "==GPIB== listen(): Timeout occurred.\n" );
#endif

    return ret;
}



/*==========================================================================
 * uPD7210 chip initialize
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: int	gpib error code
 *--------------------------------------------------------------------------
*/
static int  upd7210_initialize( GPIB_SOFTC *psc )
{
    aux_command( psc, CHIP_RESET );
    interrupt_mask1( psc, 0x03 );
    interrupt_mask2( psc, 0x08 );	/* enable interrupt only DI,DO and CO */
    address( psc, psc->myaddress );
    address( psc, 0xe0 );		/* disable minor talk address */
    auxiliary_mode( psc, 0x20|T1679 );	/* Phase shift delay counter */
    auxiliary_mode( psc, 0x80|0x00 );	/* Aux(A) Normal handshake, disable EOS */
    auxiliary_mode( psc, 0xa0|MODE_INTLVL|MODE_T1);	/* Aux (B) */
    auxiliary_mode( psc, 0xc0|0x00 );	/* Aux (E) */

    switch( psc->flags & 0x03 )	/* Device driver working mode */
    {
	case 1: address_mode( psc, TRM01|0x80 ); break; /* talkonly */
	case 2: address_mode( psc, TRM01|0x40 ); break; /* listenonly */
	default:address_mode( psc, TRM01|0x01 ); break; /* addressable mode 1 */
    }

    aux_command( psc, IMMEDIATE_EXECUTE_PON );

    if( FLAGS_IS_CONTROLLER(psc->flags) )	/* GPIB Bus Initialize */
    {
	gpib_ifc( psc );
	gpib_remote( psc );
    }
    return 0;
}



/*==========================================================================
 * Device attach common routine.
 *--------------------------------------------------------------------------
 * arg: psc
 * ret: none
 *--------------------------------------------------------------------------
 *
*/
void
gpib_attach_common( GPIB_SOFTC *psc )
{
    int minor, i;

    /*
     * initialize softc structure
     */
    psc->myaddress = MY_ADDRESS;
    psc->inuse = 0;
    psc->state = STATE_NONE;
    psc->timeout = TIMEOUT;
    psc->default_delimiter = DELIMITER;
    psc->statusbyte = 0;

    /*
     * driver chip initialize
     */
    upd7210_initialize( psc );

    /*
     * make device
     */
    minor = psc->unit*32;
    make_dev( &gpib_cdevsw, minor, UID_ROOT, GID_WHEEL, 0666, DRIVER_NAME "%d", psc->unit );
    for( i = 0; i < GPIB_MAX_ADDRESS; i++ )
    {
	make_dev( &gpib_cdevsw, minor+i, UID_ROOT, GID_WHEEL, 0666, DRIVER_NAME "%da%d", psc->unit, i );
    }
}
