/************************************************/
/* uPD7210 driver				*/
/*----------------------------------------------*/
/* cbus probe & attach module			*/
/*						*/
/* This file is completely in public domain.	*/
/************************************************/
/*
 $Id: upd7210_cbus.c,v 1.1 2003/11/03 12:40:48 hirohito Exp $
*/

/***** Feature test switches ************************************************/
/***** System headers *******************************************************/
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/module.h>
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>


/***** Local headers ********************************************************/
#include <dev/upd7210/upd7210.h>
#include <dev/upd7210/upd7210_var.h>


/***** Constat values *******************************************************/
/***** Macros ***************************************************************/
/***** Typedefs *************************************************************/
/***** Function prototypes **************************************************/
static int  gpib_isa_probe( device_t dev );
static int  gpib_isa_attach( device_t dev );


/***** device driver information ********************************************/
static device_method_t gpib_isa_methods[] =
{   /* Device interface */
    DEVMETHOD( device_probe,    gpib_isa_probe ),
    DEVMETHOD( device_attach,   gpib_isa_attach ),
    { 0, 0 }
};

static driver_t gpib_isa_driver =
{
    DRIVER_NAME,
    gpib_isa_methods,
    sizeof(GPIB_SOFTC),
};


/***** Local variables ******************************************************/
/***** Global variables *****************************************************/
/***** Local functions ******************************************************/
/***** Global functions *****************************************************/
/*==========================================================================
 * Device probe
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int	zero, if no error.
 *--------------------------------------------------------------------------
 * devؤ֤򸡽Ф롣
*/
static int
gpib_isa_probe( device_t dev )
{
    GPIB_SOFTC *psc;
    int ret = 0;

    struct resource *res_dsw;
    int rid_dsw = 1;
    int dipsw;
    int tbl_irq[] = { 3, 10, 12, 13 };

#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:CBUS== probe()\n" );
    device_printf( dev, "IOPORT=0x%x, IRQ=%d\n",
		   (int)bus_get_resource_start( dev, SYS_RES_IOPORT, 0 ),
		   (int)bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) );
#endif

    /*
     * check configuration
     */
    if( bus_get_resource_start( dev, SYS_RES_IOPORT, 0 ) == 0 ) return ENXIO;
    if( bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) == 0 ) return ENXIO;

    /*
     *  allocate io-port resource
     */
    psc = device_get_softc( dev );
    bzero( psc, sizeof(GPIB_SOFTC) );
    psc->rid = 0;
    psc->res = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid, 0, ~0,
				   USED_IO_SPACE, RF_ACTIVE );
    if( psc->res == NULL ) return ENXIO;
    psc->bst = rman_get_bustag( psc->res );
    psc->bsh = rman_get_bushandle( psc->res );
    if( bus_set_resource( dev, SYS_RES_IOPORT, 1, 0x99, rid_dsw ) != 0 )
    {
	device_printf( dev, "Can't allocate I/O port 0x99\n" );
	return ENXIO;
    }

    /*
     * probe
     * ɥ쥹쥸 0x1f򥻥åȤƤߤơɤ뤫å
     */
    aux_command( psc, CHIP_RESET );
    address( psc, 0x1f );
    if( address0( psc ) != 0x1f ) ret = ENXIO;		/* device not installed */

    /*
     * read dip switch setting.
     */
    res_dsw = bus_alloc_resource( dev, SYS_RES_IOPORT, &rid_dsw, 0x99, 0x99, 1, RF_ACTIVE );
    if( res_dsw == NULL )
    {
	device_printf( dev, "can't alloc I/O(0x99) resource\n" );
	return ENXIO;
    }
    dipsw = bus_space_read_1( rman_get_bustag( res_dsw ), rman_get_bushandle( res_dsw ), 0 );
    device_printf( dev, "dip sw = 0x%x\n", dipsw );
    if( tbl_irq[(dipsw >> 6) & 0x03] != bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) )
    {
	device_printf( dev, "Illegal IRQ setting. board=%d, config=%d\n",
		       tbl_irq[(dipsw >> 6) & 0x03],
		       (int)bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) );
	return ENXIO;
    }

    /*
     * release resource
     */
    bus_release_resource( dev, SYS_RES_IOPORT, psc->rid, psc->res );
    bus_release_resource( dev, SYS_RES_IOPORT, rid_dsw, res_dsw );

#ifdef GPIB_DEBUG
    device_printf( dev, "probed status %d\n", ret );
#endif

    return ret;
}



/*==========================================================================
 * Device attach
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int	
 *--------------------------------------------------------------------------
 *
*/
static int
gpib_isa_attach( device_t dev )
{
    GPIB_SOFTC *psc;

#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:CBUS== attach()\n" );
#endif

    /*
     * allocate resource
     */
    psc = device_get_softc( dev );
    psc->unit = device_get_unit( dev );
    psc->flags = device_get_flags( dev );

    /* IO_PORT */
    psc->rid = 0;
    psc->res = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid, 0, ~0,
				   USED_IO_SPACE, RF_ACTIVE );
    if( psc->res == NULL ) return ENXIO;
    psc->bst = rman_get_bustag( psc->res );
    psc->bsh = rman_get_bushandle( psc->res );

    /* IRQ */
    psc->irqid = 0;
    psc->irq = bus_alloc_resource( dev, SYS_RES_IRQ, &psc->irqid, 0, ~0,
				   1, RF_SHAREABLE | RF_ACTIVE );
    if( psc->irq == NULL ) return ENXIO;
    bus_setup_intr( dev, psc->irq, INTR_TYPE_MISC, gpib_interrupt, psc, &psc->irqhandle );

    /* etc */
    gpib_attach_common( psc );

    device_printf( dev, "type PC-9801-29\n" );
#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:CBUS== psc=%p res=%p bst=0x%x bsh=0x%x\n", psc, psc->res, psc->bst, psc->bsh );
#endif
    return 0;
}


DRIVER_MODULE( gpib, isa, gpib_isa_driver, gpib_devclass, 0, 0 );
