/************************************************/
/* uPD7210 driver				*/
/*----------------------------------------------*/
/* isa probe & attach module			*/
/*						*/
/* This file is completely in public domain.	*/
/************************************************/
/*
 $Id: upd7210_isa.c,v 1.2 2003/11/03 12:40:48 hirohito Exp $
*/

/***** Feature test switches ************************************************/
/***** System headers *******************************************************/
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/module.h>
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>


/***** Local headers ********************************************************/
#include <dev/upd7210/upd7210.h>
#include <dev/upd7210/upd7210_var.h>


/***** Constat values *******************************************************/
/***** Macros ***************************************************************/
/***** Typedefs *************************************************************/
/***** Function prototypes **************************************************/
static int  gpib_isa_probe( device_t dev );
static int  gpib_isa_attach( device_t dev );


/***** device driver information ********************************************/
static device_method_t gpib_isa_methods[] =
{   /* Device interface */
    DEVMETHOD( device_probe,    gpib_isa_probe ),
    DEVMETHOD( device_attach,   gpib_isa_attach ),
    { 0, 0 }
};

static driver_t gpib_isa_driver =
{
    DRIVER_NAME,
    gpib_isa_methods,
    sizeof(GPIB_SOFTC),
};


/***** Local variables ******************************************************/
/***** Global variables *****************************************************/
/***** Local functions ******************************************************/
/***** Global functions *****************************************************/
/*==========================================================================
 * Device probe
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int	zero, if no error.
 *--------------------------------------------------------------------------
 * devؤ֤򸡽Ф롣
*/
static int
gpib_isa_probe( device_t dev )
{
    GPIB_SOFTC *psc;
    int ret = 0;

#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:ISA== probe()\n" );
    device_printf( dev, "IOPORT=0x%x, IRQ=%d\n",
		   (int)bus_get_resource_start( dev, SYS_RES_IOPORT, 0 ),
		   (int)bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) );
#endif

    /*
     * check configuration
     */
    if( bus_get_resource_start( dev, SYS_RES_IOPORT, 0 ) == 0 ) return ENXIO;
    if( bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) == 0 ) return ENXIO;

    /*
     *  allocate io-port resource
     */
    psc = device_get_softc( dev );
    bzero( psc, sizeof(GPIB_SOFTC) );
    psc->rid = 0;
    psc->res = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid, 0, ~0,
				   USED_IO_SPACE, RF_ACTIVE );
    if( psc->res == NULL ) return ENXIO;
    psc->bst = rman_get_bustag( psc->res );
    psc->bsh = rman_get_bushandle( psc->res );
#if defined(GPIB_CONTEC_PC)
    psc->rid2 = 1;
    if( bus_set_resource( dev, SYS_RES_IOPORT, psc->rid2, 0x02f0, 8 ) != 0 )
    {
	device_printf( dev, "Can't allocate I/O port 0x2f0-0x2f7\n" );
	return ENXIO;
    }
#endif

    /*
     * probe
     * ɥ쥹쥸 0x1f򥻥åȤƤߤơɤ뤫å
     */
    aux_command( psc, CHIP_RESET );
    address( psc, 0x1f );
    if( address0( psc ) != 0x1f ) ret = ENXIO;		/* device not installed */

    /*
     * release resource
     */
    bus_release_resource( dev, SYS_RES_IOPORT, psc->rid, psc->res );

#ifdef GPIB_DEBUG
    device_printf( dev, "probed status %d\n", ret );
#endif

    return ret;
}



/*==========================================================================
 * Device attach
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int	
 *--------------------------------------------------------------------------
 *
*/
static int
gpib_isa_attach( device_t dev )
{
    GPIB_SOFTC *psc;
    int irq_no;

#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:ISA== attach()\n" );
#endif

    /*
     * allocate resource
     */
    psc = device_get_softc( dev );
    psc->unit = device_get_unit( dev );
    psc->flags = device_get_flags( dev );

    /* IO_PORT */
    psc->rid = 0;
    psc->res = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid, 0, ~0,
				   USED_IO_SPACE, RF_ACTIVE );
    if( psc->res == NULL ) return ENXIO;
    psc->bst = rman_get_bustag( psc->res );
    psc->bsh = rman_get_bushandle( psc->res );

    /* IRQ */
    irq_no = bus_get_resource_start( dev, SYS_RES_IRQ, 0 );
    psc->irqid = 0;
    psc->irq = bus_alloc_resource( dev, SYS_RES_IRQ, &psc->irqid, 0, ~0,
				   1, RF_SHAREABLE | RF_ACTIVE );
    if( psc->irq == NULL ) return ENXIO;
    bus_setup_intr( dev, psc->irq, INTR_TYPE_MISC, gpib_interrupt, psc, &psc->irqhandle );

    /* etc */
    gpib_attach_common( psc );

#if defined(GPIB_CONTEC_PC)
    device_printf( dev, "type CONTEC GP-IB(PC)\n" );
    /* (note)
     * contec GP-IB(PC) ϡȼ˳ߵĥ쥸äƤ뤫顢
     * Υ꥽Ƥ
     */
    psc->rid2 = 1;
    psc->res2 = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid2, 0, ~0, 1, RF_ACTIVE );
    if( psc->res2 == NULL ) return ENXIO;
    psc->bst2 = rman_get_bustag( psc->res2 );
    psc->bsh2 = rman_get_bushandle( psc->res2 );
    psc->int_mask_ofs = irq_no;
    bus_space_write_1( psc->bst2, psc->bsh2, psc->int_mask_ofs, 0 );

#elif defined(GPIB_CONTEC_PCL)
    device_printf( dev, "type CONTEC GP-IB(PC)L\n" );
    /* (note)
     * IRQϡ١ɥ쥹+1ΥݡȤ˽񤭹ߤ뤳ȤǹԤʤ褦
     */
    bus_space_write_1( psc->bst, psc->bsh, 0x01, irq_no );
#endif

#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:ISA== psc=%p res=%p bst=0x%x bsh=0x%x\n", psc, psc->res, psc->bst, psc->bsh );
#if defined(GPIB_CONTEC_PC)
    device_printf( dev, "==GPIB:ISA== psc=%p res=%p bst=0x%x bsh=0x%x\n", psc, psc->res2, psc->bst2, psc->bsh2 );
#endif
#endif
    return 0;
}


DRIVER_MODULE( gpib, isa, gpib_isa_driver, gpib_devclass, 0, 0 );
