/************************************************/
/* uPD7210 driver				*/
/*----------------------------------------------*/
/* pccard probe & attach module			*/
/*						*/
/* This file is completely in public domain.	*/
/************************************************/
/*
 $Id: upd7210_pccard.c,v 1.2 2003/11/03 12:40:49 hirohito Exp $
*/

/***** Feature test switches ************************************************/
/***** System headers *******************************************************/
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/module.h>
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>
#include <dev/pccard/pccardvar.h>
#include <pccard/cardinfo.h>
#include <pccard/slot.h>


/***** Local headers ********************************************************/
#include <dev/upd7210/upd7210.h>
#include <dev/upd7210/upd7210_var.h>


/***** Constat values *******************************************************/
/***** Macros ***************************************************************/
/***** Typedefs *************************************************************/
/***** Function prototypes **************************************************/
static int  gpib_pccard_probe( device_t dev );
static int  gpib_pccard_attach( device_t dev );
static int  gpib_pccard_detach( device_t dev );


/***** device driver information ********************************************/
static device_method_t gpib_pccard_methods[] =
{   /* Device interface */
    DEVMETHOD( device_probe,    gpib_pccard_probe ),
    DEVMETHOD( device_attach,   gpib_pccard_attach ),
    DEVMETHOD( device_detach,	gpib_pccard_detach),
    { 0, 0 }
};

static driver_t gpib_pccard_driver =
{
    DRIVER_NAME,
    gpib_pccard_methods,
    sizeof(GPIB_SOFTC),
};


/***** Local variables ******************************************************/
/***** Global variables *****************************************************/
/***** Local functions ******************************************************/
/***** Global functions *****************************************************/
/*==========================================================================
 * Device detach
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int
 *--------------------------------------------------------------------------
*/
static int
gpib_pccard_detach( device_t dev )
{
#ifdef GPIB_DEBUG
    printf( "==GPIB:PCCARD== detach()\n" );
#endif
    GPIB_SOFTC *psc = device_get_softc( dev );
    /*
      release resources
    */

    return 0;
}
    


/*==========================================================================
 * Device probe
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int
 *--------------------------------------------------------------------------
*/
static int
gpib_pccard_probe( device_t dev )
{
#ifdef GPIB_DEBUG
    printf( "==GPIB:PCCARD== probe()\n" );
#endif

    if( pccard_get_devid( dev ) == 0x81141221 )
    {
	device_set_desc( dev, "CONTEC GP-IB(PCCARD)L" );
	return 0;
    }

    return ENXIO;
}



/*==========================================================================
 * Device attach
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int	
 *--------------------------------------------------------------------------
 *
*/
static int  gpib_pccard_attach( device_t dev )
{
    GPIB_SOFTC *psc;
#ifdef GPIB_DEBUG
    printf( "==GPIB:PCCARD== attach()\n" );
#endif

    /*
     *  allocate resource. (i/o port, irq)
     */
    psc = device_get_softc( dev );
    psc->rid = PCIR_MAPS;
    psc->res = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid, 0, ~0,
				   USED_IO_SPACE, RF_ACTIVE );
    if( psc->res == NULL ) return ENXIO;
    psc->bst = rman_get_bustag( psc->res );
    psc->bsh = rman_get_bushandle( psc->res );
    psc->unit = device_get_unit( dev );
    psc->flags = device_get_flags( dev );
    psc->irqid = 0;
    psc->irq = bus_alloc_resource( dev, SYS_RES_IRQ, &psc->irqid, 0, ~0,
				   1, RF_SHAREABLE | RF_ACTIVE );
    if( psc->irq == NULL ) return ENXIO;
    bus_setup_intr( dev, psc->irq, INTR_TYPE_MISC, gpib_interrupt, psc, &psc->irqhandle );

    gpib_attach_common( psc );
#ifdef GPIB_DEBUG
    printf( "==GPIB:PCCARD== psc=$%p, bst=$%x, bsh=$%x unit=%d flags=%d\n", psc, psc->bst, psc->bsh, psc->unit, psc->flags );
#endif

    return 0;
}


DRIVER_MODULE( gpib, pccard, gpib_pccard_driver, gpib_devclass, 0, 0 );
