/************************************************/
/* uPD7210 driver				*/
/*----------------------------------------------*/
/* pci probe & attach module			*/
/*						*/
/* This file is completely in public domain.	*/
/************************************************/
/*
 $Id: upd7210_pci.c,v 1.2 2003/11/03 12:40:49 hirohito Exp $
*/

/***** Feature test switches ************************************************/
/***** System headers *******************************************************/
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/kernel.h>
#include <sys/bus.h>
#include <sys/module.h>
#include <machine/bus.h>
#include <machine/resource.h>
#include <sys/rman.h>
#include <pci/pcireg.h>
#include <pci/pcivar.h>


/***** Local headers ********************************************************/
#include <dev/upd7210/upd7210.h>
#include <dev/upd7210/upd7210_var.h>


/***** Constat values *******************************************************/
/***** Macros ***************************************************************/
/***** Typedefs *************************************************************/
/***** Function prototypes **************************************************/
static int  gpib_pci_probe( device_t dev );
static int  gpib_pci_attach( device_t dev );


/***** device driver information ********************************************/
static device_method_t gpib_pci_methods[] =
{   /* Device interface */
    DEVMETHOD( device_probe,    gpib_pci_probe ),
    DEVMETHOD( device_attach,   gpib_pci_attach ),
    { 0, 0 }
};

static driver_t gpib_pci_driver =
{
    DRIVER_NAME,
    gpib_pci_methods,
    sizeof(GPIB_SOFTC),
};


/***** Local variables ******************************************************/
/***** Global variables *****************************************************/
/***** Local functions ******************************************************/
/***** Global functions *****************************************************/
/*==========================================================================
 * Device probe
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int
 *--------------------------------------------------------------------------
*/
static int  gpib_pci_probe( device_t dev )
{
#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:PCI== probe()\n" );
    {
	u_long start, count;
	bus_get_resource( dev, SYS_RES_IOPORT, PCIR_MAPS, &start, &count );
	device_printf( dev, "IOPORT=0x%lx-%lx, IRQ=%d\n", start, start+count,
		       (int)bus_get_resource_start( dev, SYS_RES_IRQ, 0 ) );
    }
#endif

    if( pci_get_devid( dev ) == 0x81141221 )
    {
	device_set_desc( dev, "CONTEC GP-IB(PCI)L" );
	return 0;
    }

    return ENXIO;
}



/*==========================================================================
 * Device attach
 *--------------------------------------------------------------------------
 * arg: dev
 * ret: int	
 *--------------------------------------------------------------------------
 *
*/
static int  gpib_pci_attach( device_t dev )
{
    GPIB_SOFTC *psc;
#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:PCI== attach()\n" );
#endif

    /*
     *  allocate resource. (i/o port, irq)
     */
    psc = device_get_softc( dev );
    psc->rid = PCIR_MAPS;
    psc->res = bus_alloc_resource( dev, SYS_RES_IOPORT, &psc->rid, 0, ~0,
				   USED_IO_SPACE, RF_ACTIVE );
    if( psc->res == NULL ) return ENXIO;
    psc->bst = rman_get_bustag( psc->res );
    psc->bsh = rman_get_bushandle( psc->res );
    psc->unit = device_get_unit( dev );
    psc->flags = device_get_flags( dev );
    psc->irqid = 0;
    psc->irq = bus_alloc_resource( dev, SYS_RES_IRQ, &psc->irqid, 0, ~0,
				   1, RF_SHAREABLE | RF_ACTIVE );
    if( psc->irq == NULL ) return ENXIO;
    bus_setup_intr( dev, psc->irq, INTR_TYPE_MISC, gpib_interrupt, psc, &psc->irqhandle );

    gpib_attach_common( psc );
#ifdef GPIB_DEBUG
    device_printf( dev, "==GPIB:PCI== psc=%p res=%p bst=0x%x bsh=0x%x\n", psc, psc->res, psc->bst, psc->bsh );
#endif

    return 0;
}


DRIVER_MODULE( gpib, pci, gpib_pci_driver, gpib_devclass, 0, 0 );
